using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.IO;
using System.Xml.Serialization;
using System.Threading;

namespace Wallpapers_National_Geographic
{
    public partial class Form1 : Form
    {
        int max = 26;
        public Form1()
        {
            InitializeComponent();

            //otherwise will throw an exception when calling ReportProgress method
            backgroundWorker1.WorkerReportsProgress = true;

            //mandatory. Otherwise we would get an InvalidOperationException when trying to cancel the operation
            backgroundWorker1.WorkerSupportsCancellation = true;
        }



        //This method is executed in a separate thread created by the background worker.
        //Don't try to access and UI controls here

        //this attribute will prevent the debugger to stop here if any exception is raised.
        //[System.Diagnostics.DebuggerNonUserCodeAttribute()]
        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            DateTime start = DateTime.Now;
            
            e.Result = "";
            int j = 0;

            for (int i = 1; i <= max; i++)
            {

                System.Threading.Thread.Sleep(50);
                string imgurl = "";
                if (chk2009.Checked)
                {
                    string[] wall = new string[] { "1109", "1102", "1026", "1019", "1013", "1005", "0928", "0921", "0914", "0907", "0831", "0824", "0817" };
                    for (int k = 0; k < 13; k++)
                    {
                        imgurl = "http://ngm.nationalgeographic.com/photo-contest/2009/img/wallpaper/"+wall[k]+"wallpaper-" + i.ToString() + "_1600.jpg";
                        string filename = wall[k] + "wallpaper-" + i.ToString() + "_1600.jpg";
                        backgroundWorker1.ReportProgress(0, "Downloading... " + filename);
                        int status = GetImage(imgurl, filename);
                        Thread.Sleep(50);
                        j++;
                        if (status == 1)
                        {
                            backgroundWorker1.ReportProgress(j, filename + " Downloaded");
                        }
                        else if (status == 0)
                        {
                            backgroundWorker1.ReportProgress(j, filename + " already exists");
                        }
                        else
                        {
                            backgroundWorker1.ReportProgress(j, filename + " Error occured ");
                        }
                    }

                }
                if (backgroundWorker1.CancellationPending)
                {
                    e.Cancel = true;
                    return;
                }
                if (chk2008.Checked)
                {
                    string[] wall = new string[] { "1107", "1103", "1027", "1020", "1014", "1006", "0929", "0922", "0915", "0908", "0901", "0825", "0818", "0811" };
                     for (int k = 0; k < 13; k++)
                     {
                         imgurl = "http://ngm.nationalgeographic.com/photo-contest/img/wallpaper/" + wall[k] + "wallpaper-" + i.ToString() + "_1024.jpg";
                         string filename = wall[k] + "wallpaper-" + i.ToString() + "_1024.jpg";
                         backgroundWorker1.ReportProgress(0, "Downloading... " +filename);
                         int status = GetImage(imgurl, filename);
                         Thread.Sleep(500);
                         j++;
                         if (status == 1)
                         {
                             backgroundWorker1.ReportProgress(j, filename+" Downloaded");
                         }
                         else if (status == 0)
                         {
                             backgroundWorker1.ReportProgress(j, filename+" already exists");
                         }
                         else
                         {
                             backgroundWorker1.ReportProgress(j, filename+" Error occured ");
                         }
                     }
                }
                if (backgroundWorker1.CancellationPending)
                {
                    e.Cancel = true;
                    return;
                }
            }

            TimeSpan duration = DateTime.Now - start;
            e.Result = "Files tried:"+j.ToString()+". Duration: " + duration.TotalMilliseconds.ToString() + " ms.";

        }



        //This event is raised on the main thread.
        //It is ok to access UI controls here.
        private void backgroundWorker1_ProgressChanged(object sender,
            ProgressChangedEventArgs e)
        {
            if (e.ProgressPercentage == 0)
            {
                txtOutput.AppendText(e.UserState.ToString());
                txtOutput.AppendText(Environment.NewLine);
            }
            else
            {
                progressBar1.Value = e.ProgressPercentage;
                txtOutput.AppendText(e.UserState.ToString());
                txtOutput.AppendText(Environment.NewLine);
            }
        }



        //This is executed after the task is complete whatever the task has completed: a) sucessfully, b) with error c)has been cancelled
        private void backgroundWorker1_RunWorkerCompleted(object sender,
            RunWorkerCompletedEventArgs e)
        {
            if (e.Cancelled)
            {
                MessageBox.Show("The task has been cancelled");
            }
            else if (e.Error != null)
            {
                MessageBox.Show("Error. Details: " + (e.Error as Exception).ToString());
            }
            else
            {
                MessageBox.Show("Download completed. Results: " + e.Result.ToString());
            }

        }

        private void btoCancel_Click(object sender, EventArgs e)
        {
            //notify background worker we want to cancel the operation.
            //this code doesn't actually cancel or kill the thread that is executing the job.
            backgroundWorker1.CancelAsync();
        }

        private void btoStart_Click(object sender, EventArgs e)
        {
            
            if (!chk2009.Checked && !chk2008.Checked)
            {
                MessageBox.Show("Select Year");
            }
            else if (fldrBrowser.SelectedPath.Trim() == "")
            {
                MessageBox.Show("Select folder to save images");
            }
            else
            {
                txtOutput.Text = "Downloading......" + Environment.NewLine;
                progressBar1.Maximum = 0;
                if (chk2009.Checked)
                {
                    progressBar1.Maximum = max*13;
                }

                if (chk2008.Checked)
                {
                    progressBar1.Maximum = progressBar1.Maximum + (max*14);
                }
                //btoStart.Text = "Processing....";
                backgroundWorker1.RunWorkerAsync();
                //btoStart.Text = "Start Downloading";

            }
        }

        private void btnFolder_Click(object sender, EventArgs e)
        {
            fldrBrowser.ShowDialog();
        }

        private int GetImage(string url, string filename)
        {
            try
            {
                string filePath = fldrBrowser.SelectedPath + "\\" + filename;
                if (!File.Exists(filePath))
                {
                    Stream str = null;
                    HttpWebRequest wReq = (HttpWebRequest)WebRequest.Create(url);
                    HttpWebResponse wRes = (HttpWebResponse)(wReq).GetResponse();
                    str = wRes.GetResponseStream();
                    Image img = Image.FromStream(str);
                    img.Save(fldrBrowser.SelectedPath + "\\" + filename);
                    return (1);
                }
                else
                {
                    //MessageBox.Show("a file with name \"" + filename + "\" already exists in the folder");
                    return (0);
                }

            }
            catch (Exception exp)
            {
                //MessageBox.Show(url + " -- " + exp.Message);
                return (-1);
            }
        }


    }
}